@@ -26,9 +26,11 @@
 #include <apr_file_io.h>
 #include <apr_hash.h>
 #include <apr_hooks.h>
+#include <apr_network_io.h>
 #include <apr_poll.h>
 #include <apr_pools.h>
 #include <apr_tables.h>
+#include <apr_uri.h>
 
 #define REDWAX_DECLARE_MODULE(foo) \
     module foo##_module
@@ -167,6 +169,11 @@ typedef struct redwax_tool_t {
     int dane_nxdomain;
     int dane_malformed;
     int dane_record;
+    int tls_bogus;
+    int tls_insecure;
+    int tls_nxdomain;
+    int tls_malformed;
+    int tls_record;
     int poll_work;
     int current;
     int cert_out;
@@ -249,6 +256,7 @@ typedef struct redwax_certificate_t {
     const unsigned char *der;
     apr_size_t len;
     const char *origin;
+    const char *address;
     /* ID from the input certificate */
     const unsigned char *id_der;
     apr_size_t id_len;
@@ -356,6 +364,14 @@ typedef struct redwax_dns_t redwax_dns_t;
 
 typedef apr_status_t(* redwax_dns_cb_t) (redwax_tool_t *r, redwax_dns_t *dns);
 
+typedef struct redwax_rdata_a_t {
+    apr_sockaddr_t *sockaddr;
+} redwax_rdata_a_t;
+
+typedef struct redwax_rdata_aaaa_t {
+    apr_sockaddr_t *sockaddr;
+} redwax_rdata_aaaa_t;
+
 typedef struct redwax_rdata_tlsa_t {
     const char *usage_name;
     const char *selector_name;
@@ -371,6 +387,8 @@ typedef struct redwax_rdata_t {
     unsigned char *data;
     int len;
     union {
+        redwax_rdata_a_t a;
+        redwax_rdata_a_t aaaa;
         redwax_rdata_tlsa_t tlsa;
     } rr;
 } redwax_rdata_t;
@@ -388,10 +406,19 @@ typedef struct redwax_dns_t {
     int secure;
     int bogus;
     const char *why_bogus;
+    int port;
     redwax_dns_cb_t cb;
     void *ctx;
 } redwax_dns_t;
 
+typedef struct redwax_tls_t {
+    const char *unparsed_uri;
+    apr_uri_t uri;
+    int family;
+    int type;
+    int protocol;
+} redwax_tls_t;
+
 typedef apr_status_t(* redwax_pollfd_cb_t) (redwax_tool_t *r, void *ctx, apr_pollfd_t *descriptor);
 
 typedef struct redwax_pollfd_t {
@@ -579,6 +606,22 @@ APR_DECLARE_EXTERNAL_HOOK(rt, REDWAX, apr_status_t, complete_keychain_in,
 APR_DECLARE_EXTERNAL_HOOK(rt, REDWAX, apr_status_t, process_keychain_in,
         (redwax_tool_t *r, const char *arg))
 
+/**
+ * Hook to trigger a DNS lookup to read certificates from a TLS connection.
+ *
+ * @param r The redwax-tool context.
+ */
+APR_DECLARE_EXTERNAL_HOOK(rt, REDWAX, apr_status_t, set_tls_in,
+        (redwax_tool_t *r, const char *arg))
+
+/**
+ * Hook to read certificates from a DTLS connection.
+ *
+ * @param r The redwax-tool context.
+ */
+APR_DECLARE_EXTERNAL_HOOK(rt, REDWAX, apr_status_t, process_tls_in,
+        (redwax_tool_t *r, redwax_dns_t *dns))
+
 /**
  * Hook to complete incoming certificates / intermediates / keys
  * and pass filtered results to the outgoing certificates /
