@@ -779,7 +779,7 @@ static apr_status_t redwax_keychain_search_key(redwax_tool_t *r,
     return DECLINED;
 }
 
-void redwax_add_default_keychain_hooks()
+void redwax_add_default_keychain_hooks(apr_pool_t *pool)
 {
     rt_hook_complete_keychain_in(redwax_keychain_complete_keychain_in, NULL, NULL, APR_HOOK_MIDDLE);
     rt_hook_process_keychain_in(redwax_keychain_process_keychain_in, NULL, NULL, APR_HOOK_MIDDLE);
@@ -789,7 +789,7 @@ void redwax_add_default_keychain_hooks()
 
 #else
 
-void redwax_add_default_keychain_hooks()
+void redwax_add_default_keychain_hooks(apr_pool_t *pool)
 {
 }
 
