@@ -27,7 +27,7 @@
 
 #if HAVE_SECURITY_SECURITY_H
 
-void redwax_add_default_keychain_hooks();
+void redwax_add_default_keychain_hooks(apr_pool_t *pool);
 
 #endif
 #endif /* REDWAX_KEYCHAIN_H_ */
