@@ -128,6 +128,8 @@ static apr_status_t redwax_ldns_add_tlsa_metadata(redwax_tool_t *r,
 {
     const unsigned char *der = cert->der;
 
+    /* ldns is tightly bound to openssl */
+
     X509 *x = d2i_X509(NULL, &der, cert->len);
 
     if (!x) {
@@ -178,6 +180,56 @@ static apr_status_t redwax_ldns_process_dns(redwax_tool_t *r,
 {
 
     switch (dns->rrtype) {
+    case LDNS_RR_TYPE_A: {
+
+        apr_sockaddr_t *new_sa = apr_pcalloc(r->pool, sizeof(apr_sockaddr_t));
+
+        new_sa->pool = r->pool;
+
+        new_sa->family = AF_INET;
+        new_sa->sa.sin.sin_family = AF_INET;
+
+        new_sa->sa.sin.sin_addr = *(struct in_addr *)rdata->data;
+
+        new_sa->hostname = apr_pstrdup(r->pool, dns->qname);
+
+        new_sa->salen = sizeof(struct sockaddr_in);
+        new_sa->addr_str_len = 16;
+        new_sa->ipaddr_ptr = &(new_sa->sa.sin.sin_addr);
+        new_sa->ipaddr_len = sizeof(struct in_addr);
+
+        new_sa->sa.sin.sin_port = htons(dns->port);
+        new_sa->port = dns->port;
+
+        rdata->rr.a.sockaddr = new_sa;
+
+        break;
+    }
+    case LDNS_RR_TYPE_AAAA: {
+
+        apr_sockaddr_t *new_sa = apr_pcalloc(r->pool, sizeof(apr_sockaddr_t));
+
+        new_sa->pool = r->pool;
+
+        new_sa->family = AF_INET6;
+        new_sa->sa.sin.sin_family = AF_INET6;
+
+        new_sa->sa.sin6.sin6_addr = *(struct in6_addr *)rdata->data;
+
+        new_sa->hostname = apr_pstrdup(r->pool, dns->qname);
+
+        new_sa->salen = sizeof(struct sockaddr_in6);
+        new_sa->addr_str_len = 46;
+        new_sa->ipaddr_ptr = &(new_sa->sa.sin6.sin6_addr);
+        new_sa->ipaddr_len = sizeof(struct in6_addr);
+
+        new_sa->sa.sin6.sin6_port = htons(dns->port);
+        new_sa->port = dns->port;
+
+        rdata->rr.aaaa.sockaddr = new_sa;
+
+        break;
+    }
     case LDNS_RR_TYPE_TLSA: {
 
         ldns_buffer buffer;
