@@ -692,7 +692,7 @@ static apr_status_t redwax_libical_set_calendar_alarm(redwax_tool_t *r,
     return APR_SUCCESS;
 }
 
-void redwax_add_default_libical_hooks()
+void redwax_add_default_libical_hooks(apr_pool_t *pool)
 {
     rt_hook_initialise(redwax_libical_initialise, NULL, NULL, APR_HOOK_MIDDLE);
     rt_hook_process_calendar_out(redwax_libical_process_calendar_out, NULL, NULL, APR_HOOK_MIDDLE);
@@ -702,7 +702,7 @@ void redwax_add_default_libical_hooks()
 
 #else
 
-void redwax_add_default_libical_hooks()
+void redwax_add_default_libical_hooks(apr_pool_t *pool)
 {
 }
 
