@@ -43,6 +43,7 @@
 #include <pk11pub.h>
 #include <pkcs11t.h>
 #include <secerr.h>
+#include <certdb.h>
 
 #define REDWAX_NSS_MAX HUGE_STRING_LEN
 
@@ -461,7 +462,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                 if (PORT_GetError() == SEC_ERROR_TOKEN_NOT_LOGGED_IN) {
                     rv = PK11_Authenticate(slot, PR_TRUE, &s);
                     if (rv != SECSuccess) {
-                        redwax_print_error(r, "Error: could not log in to token '%s', giving up.\n",
+                        redwax_print_error(r, "nss-out: could not log in to token '%s', giving up.\n",
                                 PK11_GetTokenName(slot));
                         apr_pool_destroy(pool);
                         return APR_EACCES;
@@ -473,7 +474,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                     }
                 }
                 if (rv != SECSuccess) {
-                    redwax_print_error(r, "Warning: could not import key to token '%s', skipping: %s\n",
+                    redwax_print_error(r, "nss-out: could not import key to token '%s', skipping: %s\n",
                             PK11_GetTokenName(slot), PR_ErrorToName(PR_GetError()));
                     continue;
                 }
@@ -491,7 +492,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
 
             x = CERT_DecodeCertFromPackage((char *)cert->der, cert->len);
             if (!x) {
-                redwax_print_error(r, "Warning: could not decode certificate to be written to '%s', skipping: %s\n",
+                redwax_print_error(r, "nss-out: could not decode certificate to be written to '%s', skipping: %s\n",
                         file, PR_ErrorToName(PR_GetError()));
                 continue;
             }
@@ -553,7 +554,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                 if (PORT_GetError() == SEC_ERROR_TOKEN_NOT_LOGGED_IN) {
                     rv = PK11_Authenticate(slot, PR_TRUE, &s);
                     if (rv != SECSuccess) {
-                        redwax_print_error(r, "Error: could not log in to token '%s', giving up.\n",
+                        redwax_print_error(r, "nss-out: could not log in to token '%s', giving up.\n",
                                 PK11_GetTokenName(slot));
                         apr_pool_destroy(pool);
                         return APR_EACCES;
@@ -569,9 +570,34 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
 
                     }
                 }
+                if (PORT_GetError() == SEC_ERROR_ADDING_CERT) {
+
+                    CERTCertificate *prev = PK11_FindCertFromNickname(label, NULL);
+
+                    if (prev) {
+
+                        rv = SEC_DeletePermCertificate(prev);
+
+                        CERT_DestroyCertificate(prev);
+
+                        if (rv != SECSuccess) {
+                            redwax_print_error(r, "nss-out: could not remove existing certificate with label '%s' from token '%s', skipping: %s\n",
+                                    label, PK11_GetTokenName(slot), PR_ErrorToName(PR_GetError()));
+                            continue;
+                        }
+
+                        if (k) {
+                            rv = PK11_ImportCertForKeyToSlot(slot, x, (char *)label, PR_TRUE, &s);
+                        }
+                        else {
+                            rv =  PK11_ImportCert(slot, x, CK_INVALID_HANDLE, label, PR_FALSE);
+                        }
+
+                    }
+                }
                 if (rv != SECSuccess) {
-                    redwax_print_error(r, "Warning: could not add certificate to token '%s', skipping.\n",
-                            PK11_GetTokenName(slot));
+                    redwax_print_error(r, "nss-out: could not add certificate '%s' with label '%s' to token '%s', skipping: %s\n",
+                            x->subjectName, label, PK11_GetTokenName(slot), PR_ErrorToName(PR_GetError()));
                     continue;
                 }
             }
@@ -594,7 +620,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
 
             x = CERT_DecodeCertFromPackage((char *)cert->der, cert->len);
             if (!x) {
-                redwax_print_error(r, "Could not decode certificate to be written to '%s', skipping: %s\n",
+                redwax_print_error(r, "nss-out: could not decode certificate to be written to '%s', skipping: %s\n",
                         file, PR_ErrorToName(PR_GetError()));
                 apr_pool_destroy(pool);
                 return APR_EINVAL;
@@ -612,7 +638,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                 if (xx) {
 
                     redwax_print_error(r,
-                            "Warning: nss-out: intermediate '%s' already exists, skipping.\n",
+                            "nss-out: intermediate '%s' already exists, skipping.\n",
                             x->subjectName);
 
                     apr_pool_cleanup_register(pool, xx, cleanup_cert,
@@ -647,7 +673,7 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                 if (PORT_GetError() == SEC_ERROR_TOKEN_NOT_LOGGED_IN) {
                     rv = PK11_Authenticate(slot, PR_TRUE, &s);
                     if (rv != SECSuccess) {
-                        redwax_print_error(r, "Error: could not log in to token '%s', giving up.\n",
+                        redwax_print_error(r, "nss-out: could not log in to token '%s', giving up.\n",
                                 PK11_GetTokenName(slot));
                         apr_pool_destroy(pool);
                         return APR_EACCES;
@@ -657,9 +683,28 @@ static apr_status_t redwax_nss_process_nss_out(redwax_tool_t *r,
                                 label, PR_FALSE);
                     }
                 }
+                if (PORT_GetError() == SEC_ERROR_ADDING_CERT) {
+
+                    CERTCertificate *prev = PK11_FindCertFromNickname(label, NULL);
+
+                    if (prev) {
+
+                        rv = SEC_DeletePermCertificate(prev);
+
+                        CERT_DestroyCertificate(prev);
+
+                        if (rv != SECSuccess) {
+                            redwax_print_error(r, "nss-out: could not remove existing intermediate certificate with label '%s' from token '%s', skipping: %s\n",
+                                    label, PK11_GetTokenName(slot), PR_ErrorToName(PR_GetError()));
+                            continue;
+                        }
+
+                        rv =  PK11_ImportCert(slot, x, CK_INVALID_HANDLE, label, PR_FALSE);
+                    }
+                }
                 if (rv != SECSuccess) {
-                    redwax_print_error(r, "Warning: could not add certificate to token '%s', skipping.\n",
-                            PK11_GetTokenName(slot));
+                    redwax_print_error(r, "nss-out: could not add intermediate certificate '%s' with label '%s' to token '%s', skipping: %s\n",
+                            x->subjectName, label, PK11_GetTokenName(slot), PR_ErrorToName(PR_GetError()));
                     continue;
                 }
             }
