@@ -27,7 +27,7 @@
 
 #if HAVE_P11_KIT_MODULES_LOAD_AND_INITIALIZE
 
-void redwax_add_default_p11kit_hooks();
+void redwax_add_default_p11kit_hooks(apr_pool_t *pool);
 
 #endif
 #endif
