@@ -41,12 +41,9 @@ module unbound_module;
 
 typedef struct {
     struct ub_ctx *ctx;
-    apr_pollfd_t socket_read;
-    redwax_pollfd_t read_ctx;
-    apr_pollfd_t socket_write;
-    redwax_pollfd_t write_ctx;
+    apr_pollfd_t socket_poll;
+    redwax_pollfd_t poll_ctx;
     apr_status_t status;
-    int socket_read_work;
     int dns_work;
     int dns_server_set;
     int dns_trust_anchor_set;
@@ -162,6 +159,7 @@ static apr_status_t redwax_unbound_set_tlsa(redwax_tool_t *r, const char *arg)
             dns->rrtype = 52 /* TYPE TLSA */;
             dns->rrclass = 1 /* CLASS IN (internet) */;
             dns->cb = rt_run_process_tlsa;
+            dns->port = uri.port;
 
             qname = apr_array_push(r->tlsa_qnames);
             *qname = tlsa;
@@ -201,6 +199,7 @@ static apr_status_t redwax_unbound_set_tlsa(redwax_tool_t *r, const char *arg)
                 dns->rrtype = 52 /* TYPE TLSA */;
                 dns->rrclass = 1 /* CLASS IN (internet) */;
                 dns->cb = rt_run_process_tlsa;
+                dns->port = uri.port ? uri.port : ntohs(ent->s_port);
 
                 qname = apr_array_push(r->tlsa_qnames);
                 *qname = tlsa;
@@ -237,6 +236,164 @@ static apr_status_t redwax_unbound_set_smimea(redwax_tool_t *r, const char *arg)
     return APR_ENOTIMPL;
 }
 
+static apr_status_t redwax_unbound_set_tls_in(redwax_tool_t *r, const char *arg)
+{
+    apr_uri_t uri;
+
+    apr_status_t status;
+
+    int protocol = 0;
+    int type = 0;
+
+    status = apr_uri_parse(r->pool, arg, &uri);
+    if (APR_SUCCESS != status) {
+        redwax_print_error(r,
+                "Could not parse URI '%s': %pm\n", arg, &status);
+        return status;
+    }
+
+    if (!uri.hostname) {
+        redwax_print_error(r,
+                "URI '%s': hostname missing\n", arg);
+        return APR_EINVAL;
+    }
+
+    /* fixme: Use inet_pton to detect raw IP/IPv6 addresses and skip lookup */
+
+    if (!strcmp(uri.scheme, "tcp")) {
+        protocol = APR_PROTO_TCP;
+        type = SOCK_STREAM;
+    }
+    else if (!strcmp(uri.scheme, "udp")) {
+        protocol = APR_PROTO_UDP;
+        type = SOCK_DGRAM;
+    }
+    else if (!strcmp(uri.scheme, "sctp")) {
+        protocol = APR_PROTO_SCTP;
+        type= SOCK_SEQPACKET;
+    }
+
+    if (protocol) {
+
+        if (uri.port) {
+
+            redwax_dns_t *dns;
+            redwax_tls_t *tls;
+
+            tls = apr_palloc(r->pool, sizeof(redwax_tls_t));
+
+            tls->unparsed_uri = arg;
+            tls->uri = uri;
+            tls->protocol = protocol;
+            tls->type = type;
+
+            dns = apr_array_push(r->dns_requests);
+
+            dns->r = r;
+            dns->basename = uri.hostname;
+            dns->qname = uri.hostname;
+            dns->rrtype = 28 /* TYPE AAAA */;
+            dns->rrclass = 1 /* CLASS IN (internet) */;
+            dns->cb = rt_run_process_tls_in;
+            dns->ctx = tls;
+            dns->port = uri.port;
+
+            dns = apr_array_push(r->dns_requests);
+
+            dns->r = r;
+            dns->basename = uri.hostname;
+            dns->qname = uri.hostname;
+            dns->rrtype = 1 /* TYPE A */;
+            dns->rrclass = 1 /* CLASS IN (internet) */;
+            dns->cb = rt_run_process_tls_in;
+            dns->ctx = tls;
+            dns->port = uri.port;
+
+            return APR_SUCCESS;
+        }
+        else {
+            redwax_print_error(r,
+                    "URI '%s': port missing\n", arg);
+            return APR_EINVAL;
+        }
+    }
+
+    else if (uri.scheme) {
+
+        int found = 0;
+
+        setservent(1);
+
+        struct servent *ent;
+
+        while ((ent = getservent())) {
+
+            if (!strcmp(uri.scheme, ent->s_name)) {
+
+                redwax_dns_t *dns;
+                redwax_tls_t *tls;
+
+                tls = apr_palloc(r->pool, sizeof(redwax_tls_t));
+
+                /* we hard code the protocol to TCP, as the services
+                 * file contacts lots of bogus tcp/udp entries.
+                 */
+
+                tls->unparsed_uri = arg;
+                tls->uri = uri;
+                tls->protocol = APR_PROTO_TCP;
+                tls->type = SOCK_STREAM;
+
+                dns = apr_array_push(r->dns_requests);
+
+                dns->r = r;
+                dns->basename = uri.hostname;
+                dns->qname = uri.hostname;
+                dns->rrtype = 28 /* TYPE AAAA */;
+                dns->rrclass = 1 /* CLASS IN (internet) */;
+                dns->cb = rt_run_process_tls_in;
+                dns->ctx = tls;
+                dns->port = uri.port ? uri.port : ntohs(ent->s_port);
+
+                dns = apr_array_push(r->dns_requests);
+
+                dns->r = r;
+                dns->basename = uri.hostname;
+                dns->qname = uri.hostname;
+                dns->rrtype = 1 /* TYPE A */;
+                dns->rrclass = 1 /* CLASS IN (internet) */;
+                dns->cb = rt_run_process_tls_in;
+                dns->ctx = tls;
+                dns->port = uri.port ? uri.port : ntohs(ent->s_port);
+
+                found = 1;
+
+                break;
+            }
+
+        }
+
+        endservent();
+
+        if (!found) {
+            redwax_print_error(r,
+                    "URI '%s': protocol '%s' not found\n", arg, uri.scheme);
+            return APR_EINVAL;
+        }
+        else {
+            return APR_SUCCESS;
+        }
+
+    }
+
+    else {
+        redwax_print_error(r,
+                "URI '%s': scheme missing\n", arg);
+        return APR_EINVAL;
+    }
+
+}
+
 static apr_status_t redwax_unbound_filter_read_cb(redwax_tool_t *r, void *baton, apr_pollfd_t *descriptor)
 {
     unbound_config_t *config = redwax_get_module_config(r->per_module, &unbound_module);
@@ -247,9 +404,6 @@ static apr_status_t redwax_unbound_filter_read_cb(redwax_tool_t *r, void *baton,
     /* ack readcb work */
     r->poll_work--;
 
-    /* ack socket read work */
-    config->socket_read_work--;
-
     rv = ub_process(config->ctx);
 
     if (rv) {
@@ -265,19 +419,17 @@ static apr_status_t redwax_unbound_filter_read_cb(redwax_tool_t *r, void *baton,
 
     if (config->dns_work) {
 
-        status = apr_pollcb_add(r->poll, &config->socket_read);
+        /* line up next event */
+        status = apr_pollcb_add(r->poll, &config->socket_poll);
 
         if (APR_SUCCESS != status) {
             redwax_print_error(r,
-                    "Could not add read descriptor to poll: %pm\n", &status);
+                    "unbound: could not add descriptor to poll: %pm\n", &status);
             return status;
         }
 
         /* req readcb work */
         r->poll_work++;
-
-        /* req socket read work */
-        config->socket_read_work++;
     }
 
     return status;
@@ -361,42 +513,44 @@ static apr_status_t redwax_unbound_filter_write_cb(redwax_tool_t *r, void *baton
     /* req dns lookup work */
     config->dns_work++;
 
-    if (!config->socket_read_work) {
-
-        status = apr_pollcb_add(r->poll, &config->socket_read);
-
-        if (APR_SUCCESS != status) {
-            redwax_print_error(r,
-                    "Could not add read descriptor to poll: %pm\n", &status);
-            return status;
-        }
-
-        /* req readcb work */
-        r->poll_work++;
-
-        /* req socket read work */
-        config->socket_read_work++;
-
+    /* we are done sending requests, poll for responses only */
+    if (!r->dns_requests->nelts) {
+        config->socket_poll.reqevents = APR_POLLIN;
     }
 
-    if (r->dns_requests->nelts) {
-
-        status = apr_pollcb_add(r->poll, &config->socket_write);
-
-        if (APR_SUCCESS != status) {
-            redwax_print_error(r,
-                    "Could not add write descriptor to poll: %pm\n", &status);
-            return status;
-        }
-
-        /* req writecb work */
-        r->poll_work++;
+    /* line up next event */
+    status = apr_pollcb_add(r->poll, &config->socket_poll);
 
+    if (APR_SUCCESS != status) {
+        redwax_print_error(r,
+                "Could not add read descriptor to poll: %pm\n", &status);
+        return status;
     }
 
+    /* req readcb work */
+    r->poll_work++;
+
     return status;
 }
 
+static apr_status_t redwax_unbound_filter_cb(redwax_tool_t *r, void *baton, apr_pollfd_t *descriptor)
+{
+    if (descriptor->rtnevents & APR_POLLIN) {
+        /* process reads first if available to relieve pressure */
+        return redwax_unbound_filter_read_cb(r, baton, descriptor);
+    }
+    else if (descriptor->rtnevents & APR_POLLOUT) {
+        /* do writes if needed */
+        return redwax_unbound_filter_write_cb(r, baton, descriptor);
+    }
+    else {
+        /* errors are handled by ub_process() */
+        return redwax_unbound_filter_read_cb(r, baton, descriptor);
+    }
+
+    return APR_SUCCESS;
+}
+
 static apr_status_t redwax_unbound_filter_poll(redwax_tool_t *r)
 {
     unbound_config_t *config;
@@ -458,25 +612,17 @@ static apr_status_t redwax_unbound_filter_poll(redwax_tool_t *r)
         return status;
     }
 
-    /* set up the read callback */
-    config->read_ctx.cb = redwax_unbound_filter_read_cb;
+    /* set up the read/write callback */
 
-    /* set up read descriptor */
-    config->socket_read.desc_type = APR_POLL_SOCKET;
-    config->socket_read.reqevents = APR_POLLIN;
-    config->socket_read.desc.s = s;
-    config->socket_read.client_data = &config->read_ctx;
+    config->poll_ctx.cb = redwax_unbound_filter_cb;
 
-    /* set up the write callback */
-    config->write_ctx.cb = redwax_unbound_filter_write_cb;
+    /* set up read/write descriptor */
+    config->socket_poll.desc_type = APR_POLL_SOCKET;
+    config->socket_poll.reqevents = APR_POLLIN | APR_POLLOUT;
+    config->socket_poll.desc.s = s;
+    config->socket_poll.client_data = &config->poll_ctx;
 
-    /* set up write descriptor */
-    config->socket_write.desc_type = APR_POLL_SOCKET;
-    config->socket_write.reqevents = APR_POLLOUT;
-    config->socket_write.desc.s = s;
-    config->socket_write.client_data = &config->write_ctx;
-
-    status = apr_pollcb_add(r->poll, &config->socket_write);
+    status = apr_pollcb_add(r->poll, &config->socket_poll);
 
     if (APR_SUCCESS != status) {
         redwax_print_error(r,
@@ -497,6 +643,7 @@ void redwax_add_default_unbound_hooks(apr_pool_t *pool)
     rt_hook_set_dns_trust_anchor(redwax_unbound_set_dns_trust_anchor, NULL, NULL, APR_HOOK_MIDDLE);
     rt_hook_set_tlsa(redwax_unbound_set_tlsa, NULL, NULL, APR_HOOK_MIDDLE);
     rt_hook_set_smimea(redwax_unbound_set_smimea, NULL, NULL, APR_HOOK_MIDDLE);
+    rt_hook_set_tls_in(redwax_unbound_set_tls_in, NULL, NULL, APR_HOOK_MIDDLE);
     rt_hook_filter_poll(redwax_unbound_filter_poll, NULL, NULL, APR_HOOK_MIDDLE);
 }
 
