@@ -31,6 +31,6 @@
 
 #define REDWAX_DEFAULT_ROOT_KEY "/opt/local/etc/unbound/root.key"
 
-void redwax_add_default_unbound_hooks();
+void redwax_add_default_unbound_hooks(apr_pool_t *pool);
 
 #endif /* REDWAX_UNBOUND_H_ */
