@@ -1557,11 +1557,9 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
     }
     case REDWAX_FORMAT_YAML: {
 
-        void *v = (void *)val;
-
         if (object) {
 
-            if (!v) {
+            if (!val) {
 
                 const struct iovec vec[] = {
                         {"\n", 1},
@@ -1585,7 +1583,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
 
                 if (APR_SUCCESS == status) {
 
-                    void *l = strchr(v, '\n');
+                    const char *l = strchr(val, '\n');
 
                     if (l) {
 
@@ -1600,12 +1598,12 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                             const struct iovec vec[] = {
                                     {m->object_prefix, m->prefix_len},
                                     {"  ", 2},
-                                    {v, l - v + 1},
+                                    {(void *)val, l - val + 1},
                             };
 
                             status = m->wv(m->ctx, vec, 3);
 
-                            l = strchr((v = l + 1), '\n');
+                            l = strchr((val = l + 1), '\n');
                         };
 
                         if (APR_SUCCESS == status) {
@@ -1613,7 +1611,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                             const struct iovec vec[] = {
                                     {m->object_prefix, m->prefix_len},
                                     {"  ", 2},
-                                    {v, strlen(v)},
+                                    {(void *)val, strlen(val)},
                             };
 
                             status = m->wv(m->ctx, vec, 3);
@@ -1624,7 +1622,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                     else if (APR_SUCCESS == status) {
 
                         const struct iovec vec[] = {
-                                {v, strlen(v)},
+                                {(void *)val, strlen(val)},
                         };
 
                         status = m->wv(m->ctx, vec, 1);
@@ -1636,7 +1634,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
         }
         else if (array) {
 
-            if (!v) {
+            if (!val) {
 
                 const struct iovec vec[] = {
                         {"\n", 1},
@@ -1657,7 +1655,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
 
                 if (APR_SUCCESS == status) {
 
-                    void *l = strchr(v, '\n');
+                    const char *l = strchr(val, '\n');
 
                     if (l) {
 
@@ -1672,12 +1670,12 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                             const struct iovec vec[] = {
                                     {m->object_prefix, m->prefix_len},
                                     {"  ", 2},
-                                    {v, l - v + 1},
+                                    {(void *)val, l - val + 1},
                             };
 
                             status = m->wv(m->ctx, vec, 3);
 
-                            l = strchr((v = l + 1), '\n');
+                            l = strchr((val = l + 1), '\n');
                         };
 
                         if (APR_SUCCESS == status) {
@@ -1685,7 +1683,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                             const struct iovec vec[] = {
                                     {m->object_prefix, m->prefix_len},
                                     {"  ", 2},
-                                    {v, strlen(v)},
+                                    {(void *)val, strlen(val)},
                             };
 
                             status = m->wv(m->ctx, vec, 3);
@@ -1696,7 +1694,7 @@ apr_status_t redwax_metadata_add_string(redwax_metadata_t *m, const char *key, c
                     else if (APR_SUCCESS == status) {
 
                         const struct iovec vec[] = {
-                                {v, strlen(v)},
+                                {(void *)val, strlen(val)},
                         };
 
                         status = m->wv(m->ctx, vec, 1);
